@@ -104,7 +104,11 @@ def run(prg):
         if hasattr(e, 'osc_msg'):
             print >>sys.stderr, e.osc_msg
 
-        body = e.read()
+        try:
+            body = e.read()
+        except AttributeError:
+            body = ''
+
         if getattr(prg.options, 'debug', None) or \
            getattr(prg.conf, 'config', {}).get('debug', None):
             print >>sys.stderr, e.hdrs
@@ -137,6 +141,10 @@ def run(prg):
         print >>sys.stderr, e
         return 2
 
+    except oscerr.RuntimeError, e:
+        print >>sys.stderr, e.msg
+        return 1
+
     except oscerr.WorkingCopyOutdated, e:
         print >>sys.stderr, e
         return 1
@@ -165,10 +173,6 @@ def run(prg):
         print >>sys.stderr, e
         return 1
 
-    except OSError, e:
-        print >>sys.stderr, e
-        return 1
-
     except CpioError, e:
         print >>sys.stderr, e
         return 1
