@@ -8,10 +8,11 @@
 import os
 import re
 import sys
-from tempfile import NamedTemporaryFile
-from shutil import rmtree
+import shutil
+from tempfile import NamedTemporaryFile, mkdtemp
 from osc.fetch import *
 from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig, is_package_dir
+from osc.core import get_binarylist, get_binary_file
 from osc.util import rpmquery, debquery
 import osc.conf
 import oscerr
@@ -29,6 +30,8 @@ change_personality = {
             'i386':  'linux32',
             'ppc':   'powerpc32',
             's390':  's390',
+            'sparc': 'linux32',
+            'sparcv8': 'linux32',
         }
 
 can_also_build = {
@@ -47,6 +50,7 @@ can_also_build = {
              'i586':   [                'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
              'i686':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
              'x86_64': ['i686', 'i586', 'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
+             'sparc64': ['sparc64v', 'sparcv9v', 'sparcv9', 'sparcv8', 'sparc'],
              }
 
 # real arch of this machine
@@ -54,7 +58,6 @@ hostarch = os.uname()[4]
 if hostarch == 'i686': # FIXME
     hostarch = 'i586'
 
-
 class Buildinfo:
     """represent the contents of a buildinfo file"""
 
@@ -318,13 +321,38 @@ def create_deps(pkgqs):
     return depfile
 
 
-def main(opts, argv):
+trustprompt = """Would you like to ...
+0 - quit (default)
+1 - trust packages from '%(project)s' always
+2 - trust them just this time
+? """
+def check_trusted_projects(apiurl, projects):
+    trusted = config['api_host_options'][apiurl]['trusted_prj']
+    tlen = len(trusted)
+    for prj in projects:
+        if not prj in trusted:
+            print "\nThe build root needs packages from project '%s'." % prj
+            print "Note that malicious packages can compromise the build result or even your system."
+            r = raw_input(trustprompt % { 'project':prj })
+            if r == '1':
+                print "adding '%s' to ~/.oscrc: ['%s']['trusted_prj']" % (prj,apiurl)
+                trusted.append(prj)
+            elif r != '2':
+                print "Well, good good bye then :-)"
+                raise oscerr.UserAbort()
+
+    if tlen != len(trusted):
+        config['api_host_options'][apiurl]['trusted_prj'] = trusted
+        conf.config_set_option(apiurl, 'trusted_prj', ' '.join(trusted))
+
+def main(apiurl, opts, argv):
 
     repo = argv[0]
     arch = argv[1]
     build_descr = argv[2]
     xp = []
     build_root = None
+    build_uid=''
     vm_type = config['build-type']
 
     build_descr = os.path.abspath(build_descr)
@@ -378,7 +406,9 @@ def main(opts, argv):
     if opts.without:
         for o in opts.without:
             buildargs.append('--without %s' % o)
-    build_uid=''
+# FIXME: quoting
+#    if opts.define:
+#        buildargs.append('--define "%s"' % opts.define)
     if config['build-uid']:
         build_uid = config['build-uid']
     if opts.build_uid:
@@ -392,9 +422,6 @@ def main(opts, argv):
         else:
             print >>sys.stderr, 'Error: build-uid arg must be 2 colon separated numerics: "uid:gid" or "caller"'
             return 1
-# FIXME: quoting
-#    if opts.define:
-#        buildargs.append('--define "%s"' % opts.define)
     if opts.vm_type:
         vm_type = opts.vm_type
     if opts.alternative_project:
@@ -406,7 +433,6 @@ def main(opts, argv):
             pac = '_repository'
         else:
             pac = store_read_package(os.curdir)
-    apiurl = store_read_apiurl(os.curdir)
 
     # make it possible to override configuration of the rc file
     for var in ['OSC_PACKAGECACHEDIR', 'OSC_SU_WRAPPER', 'OSC_BUILD_ROOT']:
@@ -568,8 +594,6 @@ def main(opts, argv):
     if bi.release:
         buildargs.append('--release %s' % bi.release)
 
-    buildargs = ' '.join(buildargs)
-
     # real arch of this machine
     # vs.
     # arch we are supposed to build for
@@ -591,7 +615,6 @@ def main(opts, argv):
                 bi.remove_dep(name)
                 rpmlist_prefers.append((name, path))
                 print ' - %s (%s)' % (name, path)
-                continue
 
     print 'Updating cache of required packages'
 
@@ -619,15 +642,83 @@ def main(opts, argv):
                       enable_cpio = opts.cpio_bulk_download,
                       cookiejar=cookiejar)
 
+    # implicitly trust the project we are building for
+    check_trusted_projects(apiurl, [ i for i in bi.projects.keys() if not i == prj ])
+
     # now update the package cache
     fetcher.run(bi)
 
+    old_pkg_dir = None
+    if opts.oldpackages:
+        old_pkg_dir = opts.oldpackages
+        if not old_pkg_dir.startswith('/') and not opts.offline:
+            data = [ prj, pacname, repo, arch]
+            if old_pkg_dir == '_link':
+                p = osc.core.findpacs(os.curdir)[0]
+                if not p.islink():
+                    raise oscerr.WrongOptions('package is not a link')
+                data[0] = p.linkinfo.project
+                data[1] = p.linkinfo.package
+                repos = osc.core.get_repositories_of_project(apiurl, data[0])
+                # hack for links to e.g. Factory
+                if not data[2] in repos and 'standard' in repos:
+                    data[2] = 'standard'
+            elif old_pkg_dir != '' and old_pkg_dir != '_self':
+                a = old_pkg_dir.split('/')
+                for i in range(0, len(a)):
+                    data[i] = a[i]
+
+            destdir = os.path.join(config['packagecachedir'], data[0], data[2], data[3])
+            old_pkg_dir = None
+            try:
+                print "Downloading previous build from %s ..." % '/'.join(data)
+                binaries = get_binarylist(apiurl, data[0], data[2], data[3], package=data[1], verbose=True)
+            except Exception, e:
+                print "Error: failed to get binaries: %s" % str(e)
+                binaries = []
+
+            if binaries:
+                class mytmpdir:
+                    """ temporary directory that removes itself"""
+                    def __init__(self, *args, **kwargs):
+                        self.name = mkdtemp(*args, **kwargs)
+                    def cleanup(self):
+                        shutil.rmtree(self.name)
+                    def __del__(self):
+                        self.cleanup()
+                    def __exit__(self):
+                        self.cleanup()
+                    def __str__(self):
+                        return self.name
+
+                old_pkg_dir = mytmpdir(prefix='.build.oldpackages', dir=os.path.abspath(os.curdir))
+                if not os.path.exists(destdir):
+                    os.makedirs(destdir)
+            for i in binaries:
+                fname = os.path.join(destdir, i.name)
+                os.symlink(fname, os.path.join(str(old_pkg_dir), i.name))
+                if os.path.exists(fname):
+                    st = os.stat(fname)
+                    if st.st_mtime == i.mtime and st.st_size == i.size:
+                        continue
+                get_binary_file(apiurl,
+                                data[0],
+                                data[2], data[3],
+                                i.name,
+                                package = data[1],
+                                target_filename = fname,
+                                target_mtime = i.mtime,
+                                progress_meter = True)
+
+        if old_pkg_dir != None:
+            buildargs.append('--oldpackages %s' % old_pkg_dir)
+
     # Make packages from buildinfo available as repos for kiwi
     if build_type == 'kiwi':
         if not os.path.exists('repos'):
             os.mkdir('repos')
         else:
-            rmtree('repos')
+            shutil.rmtree('repos')
             os.mkdir('repos')
         for i in bi.deps:
             # project
@@ -654,27 +745,12 @@ def main(opts, argv):
                     os.symlink(sffn, tffn)
 
     if bi.pacsuffix == 'rpm':
-        if vm_type == "xen" or vm_type == "kvm" or vm_type == "lxc":
-            print 'Skipping verification of package signatures due to secure VM build'
-        elif opts.no_verify or opts.noinit or opts.offline:
+        if opts.no_verify:
             print 'Skipping verification of package signatures'
         else:
             print 'Verifying integrity of cached packages'
-            t = config['api_host_options'][apiurl]['trusted_prj']
-            for prj in bi.prjkeys:
-                if not prj in t:
-                    print "\nYou are trying to use packages from project '%s'." % prj
-                    print "Note that malicious packages can compromise your system when using chroot build enviroment."
-                    print "Use kvm or xen builds for a safe enviroment."
-# saving back to config file is complicated
-#                    r = raw_input("Would you like to trust '%s' (a)lways, (t)emorarily or (N)ever? " % prj)
-#                    if r == 'a':
-#                        config['api_host_options'][apiurl]['trusted_prj'] += prj
-#                    elif r != 't':
-#                        print "Well, good good bye then :-)"
-#                        sys.exit(1)
-
             verify_pacs([ i.fullfilename for i in bi.deps ], bi.keys)
+
     elif bi.pacsuffix == 'deb':
         if vm_type == "xen" or vm_type == "kvm" or vm_type == "lxc":
             print 'Skipping verification of package signatures due to secure VM build'
@@ -751,7 +827,7 @@ def main(opts, argv):
                     specialcmdopts,
                     bi.buildarch,
                     vm_options,
-                    buildargs,
+                    ' '.join(buildargs),
                     build_descr)
 
     if need_root:
@@ -787,7 +863,6 @@ def main(opts, argv):
 
         if opts.keep_pkgs:
             for i in b_built.splitlines() + s_built.splitlines():
-                import shutil
                 shutil.copy2(i, os.path.join(opts.keep_pkgs, os.path.basename(i)))
 
     if bi_file:
