@@ -66,7 +66,7 @@ class TrustedCertStore:
 # connection will be aborted later if it needs to
 def verify_cb(ctx, ok, store):
     if not ctx.verrs:
-	ctx.verrs = ValidationErrors()
+        ctx.verrs = ValidationErrors()
 
     try:
         if not ok:
@@ -160,6 +160,10 @@ class mySSLContext(SSL.Context):
 class myHTTPSHandler(M2Crypto.m2urllib2.HTTPSHandler):
     handler_order = 499
 
+    def __init__(self, *args, **kwargs):
+        self.appname = kwargs.pop('appname', 'generic')
+        M2Crypto.m2urllib2.HTTPSHandler.__init__(self, *args, **kwargs)
+
     # copied from M2Crypto.m2urllib2.HTTPSHandler
     # it's sole purpose is to use our myHTTPSHandler/myHTTPSProxyHandler class
     # ideally the m2urllib2.HTTPSHandler.https_open() method would be split into
@@ -168,17 +172,17 @@ class myHTTPSHandler(M2Crypto.m2urllib2.HTTPSHandler):
     def https_open(self, req):
         host = req.get_host()
         if not host:
-            raise URLError('no host given')
+            raise M2Crypto.m2urllib2.URLError('no host given: ' + req.get_full_url())
 
         # Our change: Check to see if we're using a proxy.
         # Then create an appropriate ssl-aware connection.
-        full_url = req.get_full_url() 
+        full_url = req.get_full_url()
         target_host = urlparse.urlparse(full_url)[1]
 
         if (target_host != host):
-            h = myProxyHTTPSConnection(host = host, ssl_context = self.ctx)
+            h = myProxyHTTPSConnection(host = host, appname = self.appname, ssl_context = self.ctx)
         else:
-            h = myHTTPSConnection(host = host, ssl_context = self.ctx)
+            h = myHTTPSConnection(host = host, appname = self.appname, ssl_context = self.ctx)
         # End our change
         h.set_debuglevel(self._debuglevel)
 
@@ -195,7 +199,8 @@ class myHTTPSHandler(M2Crypto.m2urllib2.HTTPSHandler):
             h.request(req.get_method(), req.get_full_url(), req.data, headers)
             r = h.getresponse()
         except socket.error, err: # XXX what error?
-            raise URLError(err)
+            err.filename = full_url
+            raise M2Crypto.m2urllib2.URLError(err)
 
         # Pick apart the HTTPResponse object to get the addinfourl
         # object initialized properly.
@@ -217,6 +222,10 @@ class myHTTPSHandler(M2Crypto.m2urllib2.HTTPSHandler):
         return resp
 
 class myHTTPSConnection(M2Crypto.httpslib.HTTPSConnection):
+    def __init__(self, *args, **kwargs):
+        self.appname = kwargs.pop('appname', 'generic')
+        M2Crypto.httpslib.HTTPSConnection.__init__(self, *args, **kwargs)
+
     def connect(self, *args):
         M2Crypto.httpslib.HTTPSConnection.connect(self, *args)
         verify_certificate(self)
@@ -228,6 +237,10 @@ class myHTTPSConnection(M2Crypto.httpslib.HTTPSConnection):
         return self.port
 
 class myProxyHTTPSConnection(M2Crypto.httpslib.ProxyHTTPSConnection):
+    def __init__(self, *args, **kwargs):
+        self.appname = kwargs.pop('appname', 'generic')
+        M2Crypto.httpslib.ProxyHTTPSConnection.__init__(self, *args, **kwargs)
+
     def _start_ssl(self):
         M2Crypto.httpslib.ProxyHTTPSConnection._start_ssl(self)
         verify_certificate(self)
@@ -318,4 +331,4 @@ Would you like to
             elif r == '9':
                 print cert.as_text()
 
-# vim: syntax=python sw=4 et
+# vim: sw=4 et
