@@ -1,14 +1,14 @@
 # Copyright 2009 Marcus Huewe <suse-tux@gmx.de>
-# 
+#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License version 2
 # as published by the Free Software Foundation;
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
@@ -19,6 +19,10 @@ import sys
 import StringIO
 import stat
 
+# workaround for python24
+if not hasattr(os, 'SEEK_SET'):
+    os.SEEK_SET = 0
+
 class ArError(Exception):
     """Base class for all ar related errors"""
     def __init__(self, fn, msg):
@@ -87,14 +91,22 @@ class Ar:
     hdr_len = 60
     hdr_pat = re.compile('^(.{16})(.{12})(.{6})(.{6})(.{8})(.{10})(.{2})', re.DOTALL)
 
-    def __init__(self, fn):
-        self.filename = fn
-        # file object: will be closed in __del__()
-        self.__file = None
+    def __init__(self, fn = None, fh = None):
+        if fn == None and fh == None:
+            raise ArError('either \'fn\' or \'fh\' must be != None')
+        if fh != None:
+            self.__file = fh
+            self.__closefile = False
+            self.filename = fh.name
+        else:
+            # file object: will be closed in __del__()
+            self.__file = None
+            self.__closefile = True
+            self.filename = fn
         self._init_datastructs()
 
     def __del__(self):
-        if self.__file:
+        if self.__file and self.__closefile:
             self.__file.close()
 
     def _init_datastructs(self):
@@ -148,9 +160,9 @@ class Ar:
             self.__file = open(self.filename, 'rb')
             try:
                 if sys.platform[:3] != 'win':
-                    self.__file = mmap.mmap(self.__file.fileno(), 0, prot=mmap.PROT_READ)
+                    self.__file = mmap.mmap(self.__file.fileno(), os.path.getsize(self.__file.name), prot=mmap.PROT_READ)
                 else:
-                    self.__file = mmap.mmap(self.__file.fileno(), 0)
+                    self.__file = mmap.mmap(self.__file.fileno(), os.path.getsize(self.__file.name))
             except EnvironmentError, e:
                 if e.errno == 19 or ( hasattr(e, 'winerror') and e.winerror == 5 ):
                     print >>sys.stderr, 'cannot use mmap to read the file, falling back to the default io'
