@@ -1,4 +1,5 @@
 import ar
+import os.path
 import re
 import tarfile
 import packagequery
@@ -13,6 +14,7 @@ class DebQuery(packagequery.PackageQuery):
 
     def __init__(self, fh):
         self.__file = fh
+        self.__path = os.path.abspath(fh.name)
         self.filename_suffix = 'deb'
         self.fields = {}
 
@@ -93,15 +95,21 @@ class DebQuery(packagequery.PackageQuery):
     def description(self):
         return self.fields['description']
 
+    def path(self):
+        return self.__path
+
     def provides(self):
         return self.fields['provides']
 
     def requires(self):
         return self.fields['depends']
 
-    def getTag(self, num):
+    def gettag(self, num):
         return self.fields.get(num, None)
 
+    def canonname(self):
+        return DebQuery.filename(self.name(), self.version(), self.release(), self.arch())
+
     @staticmethod
     def query(filename, all_tags = False, *extra_tags):
         f = open(filename, 'rb')
@@ -144,6 +152,13 @@ class DebQuery(packagequery.PackageQuery):
                     return -1
         return 0
 
+    @staticmethod
+    def filename(name, version, release, arch):
+        if release:
+            return '%s_%s-%s_%s.deb' % (name, version, release, arch)
+        else:
+            return '%s_%s_%s.deb' % (name, version, arch)
+
 if __name__ == '__main__':
     import sys
     try:
